@@ -5,6 +5,8 @@
 #   2002/03/27 : pda/jean : conception
 #   2002/05/23 : pda/jean : ajout de info-groupe
 #   2004/01/14 : pda/jean : ajout IPv6
+#   2004/08/04 : pda/jean : ajout MAC
+#   2004/08/06 : pda/jean : extension des droits sur les rseaux
 #
 
 # set debug(base)	dbname=dns-debug
@@ -76,6 +78,24 @@ set libconf(tabdomaines) {
     }
 }
 
+set libconf(tabdhcpprofil) {
+    global {
+	chars {12 normal}
+	align {left}
+	botbar {yes}
+	columns {25 75}
+    }
+    pattern DHCP {
+	vbar {yes}
+	column { }
+	vbar {no}
+	column {
+	    format {lines}
+	}
+	vbar {yes}
+    }
+}
+
 set libconf(tabmachine) {
     global {
 	chars {12 normal}
@@ -279,7 +299,7 @@ proc fermer-base {dbfd} {
 }
 
 #
-# Initialiser l'accs  DNS
+# Initialiser l'accs  DNS pour les scripts CGI
 #
 # Entre :
 #   - paramtres :
@@ -393,8 +413,83 @@ proc init-dns {nologin auth base pageerr attr form ftabvar dbfdvar loginvar tabc
 		"Formulaire non conforme aux spcifications"
 	}
     }
+}
 
-    return $dbfd
+#
+# Initialiser l'accs  DNS pour les scripts "batch"
+#
+# Entre :
+#   - paramtres :
+#	- nologin : nom du fichier test pour le mode "maintenance"
+#	- auth : paramtres d'authentification
+#	- base : nom de la base
+#	- dbfdvar : accs  la base en retour
+#	- login : login de l'utilisateur
+#	- tabcorvar : tableau contenant les caractristiques de l'utilisateur
+#		(login, password, nom, prenom, mel, tel, fax, mobile, adr,
+#			idcor, idgrp, present)
+# Sortie :
+#   - valeur de retour : message d'erreur, ou chane vide si pas d'erreur
+#   - paramtres :
+#	- dbfdvar : cf ci-dessus
+#	- tabcorvar : cf ci-dessus
+#
+# Historique
+#   2004/09/24 : pda/jean : conception
+#
+
+proc init-dns-util {nologin auth base dbfdvar login tabcorvar} {
+    upvar $dbfdvar dbfd
+    upvar $tabcorvar tabcor
+
+    #
+    # Pour le cas o on est en mode maintenance
+    #
+
+    if {[file exists $nologin]} then {
+	set fd [open $nologin r]
+	set message [read $fd]
+	close $fd
+	return "Connexion refuse.\n$message"
+    }
+
+    #
+    # Accs  la base d'authentification
+    #
+
+    set msg [::auth::init $auth]
+    if {! [string equal $msg ""]} then {
+	return "Accs  la base d'authentification impossible\n$msg"
+    }
+
+    #
+    # Accs  la base
+    #
+
+    set dbfd [ouvrir-base $base msg]
+    if {[string length $dbfd] == 0} then {
+	return "Accs  la base DNS impossible\n$msg"
+    }
+
+    #
+    # Lire toutes les caractristiques du correspondant
+    #
+
+    set msg [lire-correspondant-par-login $dbfd $login tabcor]
+    if {! [string equal $msg ""]} then {
+	return "Utilisateur '$login' : $msg"
+    }
+
+    #
+    # Si le correspondant n'est plus marqu comme "prsent" dans la base,
+    # on ne lui autorise pas l'accs  l'application
+    #
+
+    if {! $tabcor(present)} then {
+	return "Utilisateur '$login' non prsent"
+    }
+
+    return ""
 }
 
 ##############################################################################
@@ -604,6 +699,9 @@ proc lire-rr-par-ip {dbfd adr tabrr} {
 #	tabrr(nom) : nom de la machine (un seul composant du fqdn)
 #	tabrr(iddom) : l'id du domaine
 #	tabrr(domaine) : nom du domaine
+#	tabrr(mac) : l'adresse mac de la machine
+#	tabrr(iddhcpprofil) : le profil DHCP sous forme d'id, ou 0
+#	tabrr(dhcpprofil) : le nom du profil DHCP, ou "Aucun profil DHCP"
 #	tabrr(idhinfo) : le type de machine sous forme d'id
 #	tabrr(hinfo) : le type de machine sous forme de texte
 #	tabrr(commentaire) : les infos complmentaires sous forme de texte
@@ -623,27 +721,42 @@ proc lire-rr-par-ip {dbfd adr tabrr} {
 #   2002/04/19 : pda/jean : conception
 #   2002/06/02 : pda/jean : hinfo devient un index dans une table
 #   2004/02/06 : pda/jean : ajout de rolemail, adrmail et roleweb
+#   2004/08/05 : pda/jean : legere simplification et ajout de mac
+#   2005/04/08 : pda/jean : ajout de dhcpprofil
 #
 
 proc lire-rr-par-id {dbfd idrr tabrr} {
     upvar $tabrr trr
 
+    set fields {nom iddom
+	mac iddhcpprofil idhinfo commentaire respnom respmel
+	idcor date}
+
     catch {unset trr}
     set trr(idrr) $idrr
 
     set trouve 0
-    set sql "SELECT nom,iddom,idhinfo,commentaire,respnom,respmel,idcor,date \
-				FROM rr \
-				WHERE idrr = $idrr"
+    set columns [join $fields ", "]
+    set sql "SELECT $columns FROM rr WHERE idrr = $idrr"
     pg_select $dbfd $sql tab {
 	set trouve 1
-	foreach v {nom iddom idhinfo commentaire respnom respmel idcor date} {
+	foreach v $fields {
 	    set trr($v) $tab($v)
 	}
     }
 
     if {$trouve} then {
 	set trr(domaine) ""
+	if {[string equal $trr(iddhcpprofil) ""]} then {
+	    set trr(iddhcpprofil) 0
+	    set trr(dhcpprofil) "Aucun profil"
+	} else {
+	    set sql "SELECT nom FROM dhcpprofil
+				WHERE iddhcpprofil = $trr(iddhcpprofil)"
+	    pg_select $dbfd $sql tab {
+		set trr(dhcpprofil) $tab(nom)
+	    }
+	}
 	set sql "SELECT texte FROM hinfo WHERE idhinfo = $trr(idhinfo)"
 	pg_select $dbfd $sql tab {
 	    set trr(hinfo) $tab(texte)
@@ -984,7 +1097,9 @@ proc supprimer-rr-si-orphelin {dbfd idrr} {
 #	- dbfd : accs  la base
 #	- nom : nom du RR  crer (la syntaxe doit tre dj conforme  la RFC)
 #	- iddom : id du domaine du RR
-#	- idhinfo : HINFO ou -1
+#	- mac : adresse MAC, ou vide
+#	- iddhcpprofil : id du profil DHCP, ou 0
+#	- idhinfo : HINFO ou chane vide (le dfaut est pris dans la base)
 #	- comment : les infos complmentaires sous forme de texte
 #	- respnom : le nom+prnom du responsable
 #	- respmel : le ml du responsable
@@ -999,12 +1114,19 @@ proc supprimer-rr-si-orphelin {dbfd idrr} {
 #
 # Historique
 #   2004/02/13 : pda/jean : conception
+#   2004/08/05 : pda/jean : ajout mac
+#   2004/10/05 : pda      : changement du format de date
+#   2005/04/08 : pda/jean : ajout dhcpprofil
 #
 
-proc ajouter-rr {dbfd nom iddom idhinfo comment respnom respmel idcor tabrr} {
+proc ajouter-rr {dbfd nom iddom mac iddhcpprofil idhinfo comment respnom respmel idcor tabrr} {
     upvar $tabrr trr
 
-    set date [clock seconds]
+    if {[string equal $mac ""]} then {
+	set qmac NULL
+    } else {
+	set qmac "'[::pgsql::quote $mac]'"
+    }
     set qcomment [::pgsql::quote $comment]
     set qrespnom [::pgsql::quote $respnom]
     set qrespmel [::pgsql::quote $respmel]
@@ -1014,16 +1136,23 @@ proc ajouter-rr {dbfd nom iddom idhinfo comment respnom respmel idcor tabrr} {
 	set hinfodef "idhinfo,"
 	set hinfoval "$idhinfo, "
     }
+    if {$iddhcpprofil == 0} then {
+	set iddhcpprofil NULL
+    }
     set sql "INSERT INTO rr
 		    (nom, iddom,
+			mac,
+			iddhcpprofil,
 			$hinfodef
 			commentaire, respnom, respmel,
-			idcor, date)
+			idcor)
 		VALUES
 		    ('$nom', $iddom,
+			$qmac,
+			$iddhcpprofil,
 			$hinfoval
 			'$qcomment', '$qrespnom', '$qrespmel',
-			$idcor, $date)
+			$idcor)
 		    "
     if {[::pgsql::execsql $dbfd $sql msg]} then {
 	set msg ""
@@ -1050,11 +1179,12 @@ proc ajouter-rr {dbfd nom iddom idhinfo comment respnom respmel idcor tabrr} {
 #
 # Historique
 #   2002/05/03 : pda/jean : conception
+#   2004/10/05 : pda      : changement du format de date
 #
 
 proc touch-rr {dbfd idrr idcor} {
-    set date [clock seconds]
-    set sql "UPDATE rr SET idcor = $idcor, date = $date WHERE idrr = $idrr"
+    set date [clock format [clock seconds]]
+    set sql "UPDATE rr SET idcor = $idcor, date = '$date' WHERE idrr = $idrr"
     if {[::pgsql::execsql $dbfd $sql msg]} then {
        set msg ""
     } else {
@@ -1067,6 +1197,50 @@ proc touch-rr {dbfd idrr idcor} {
 # Vrifications syntaxiques
 ##############################################################################
 
+#
+# Valide la syntaxe d'un FQDN complet au sens de la RFC 1035
+# largie pour accepter les chiffres en dbut de nom.
+#
+# Entre :
+#   - paramtres :
+#	- dbfd : accs  la base
+#	- fqdn : le nom  tester
+#	- nomvar : contiendra en retour le nom de host
+#	- domvar : contiendra en retour le domaine de host
+#	- iddomvar : contiendra en retour l'id du domaine
+# Sortie :
+#   - valeur de retour : chane vide (ok) ou non vide (message d'erreur)
+#   - paramtre nom : le nom trouv
+#   - paramtre dom : le domaine trouv
+#   - paramtre iddom : l'id du domaine trouv, ou -1 si erreur
+#
+# Historique
+#   2004/09/21 : pda/jean : conception
+#   2004/09/29 : pda/jean : ajout paramtre domvar
+#
+
+proc syntaxe-fqdn {dbfd fqdn nomvar domvar iddomvar} {
+    upvar $nomvar nom
+    upvar $domvar dom
+    upvar $iddomvar iddom
+
+    if {! [regexp {^([^\.]+)\.(.*)$} $fqdn bidon nom dom]} then {
+	return "FQDN invalide ($fqdn)"
+    }
+
+    set msg [syntaxe-nom $nom]
+    if {! [string equal $msg ""]} then {
+	return $msg
+    }
+
+    set iddom [lire-domaine $dbfd $dom]
+    if {$iddom < 0} then {
+	return "Domaine '$dom' invalide"
+    }
+
+    return ""
+}
+
 #
 # Valide la syntaxe d'un nom (partie de FQDN) au sens de la RFC 1035
 # largie pour accepter les chiffres en dbut de nom.
@@ -1098,13 +1272,15 @@ proc syntaxe-nom {nom} {
     return $m
 }
 
+
+
 #
-# Valide la syntaxe d'une adresse IPv4
+# Valide la syntaxe d'une adresse IPv4 ou IPv6
 #
 # Entre :
 #   - paramtres :
 #	- adr : l'adresse  tester
-#	- type : "inet" ou "cidr"
+#	- type : "inet" ou "cidr" ou "macaddr" ou "inet4"
 # Sortie :
 #   - valeur de retour : chane vide (ok) ou non vide (message d'erreur)
 #
@@ -1113,19 +1289,103 @@ proc syntaxe-nom {nom} {
 #   2002/05/06 : pda/jean : ajout du type cidr
 #   2002/05/23 : pda/jean : reconnaissance des cas cidr simplifis (a.b/x)
 #   2004/01/09 : pda/jean : ajout du cas IPv6 et simplification radicale
+#   2004/10/08 : pda/jean : ajout du cas inet4
+#   2004/10/20 : jean     : interdit le / pour autre chose que le type cidr
 #
 
 proc syntaxe-ip {dbfd adr type} {
+
+    switch $type {
+	inet4 {
+	    set type "inet"
+	    set fam  4
+	}
+	default {
+	    set fam ""
+	}
+    }
     set adr [::pgsql::quote $adr]
     set sql "SELECT $type\('$adr'\) ;"
+    set r ""
     if {[::pgsql::execsql $dbfd $sql msg]} then {
-	set r ""
+	if {! [string equal $fam ""]} then {
+	    pg_select $dbfd "SELECT family ('$adr') AS fam" tab {
+		if {$tab(fam) != $fam} then {
+		    set r "'$adr' n'est pas une adresse IPv$fam"
+		}
+	    }
+	}
+	if {! [string equal $type "cidr"]} then {
+	    if {[regexp {/}  $adr ]} then {
+		set r "Le caractre '/' est interdit dans l'adresse"
+	    }
+	}
     } else {
 	set r "Syntaxe invalide pour '$adr'"
     }
     return $r
 }
 
+#
+# Valide la syntaxe d'une adresse MAC
+#
+# Entre :
+#   - paramtres :
+#	- adr : l'adresse  tester
+# Sortie :
+#   - valeur de retour : chane vide (ok) ou non vide (message d'erreur)
+#
+# Historique
+#   2004/08/04 : pda/jean : conception
+#
+
+proc syntaxe-mac {dbfd mac} {
+    return [syntaxe-ip $dbfd $mac "macaddr"]
+}
+
+#
+# Valide un identificateur de profil DHCP
+#
+# Entre :
+#   - paramtres :
+#	- dbfd : accs  la base
+#	- iddhcpprofil : chane de caractre reprsentant l'id, ou 0
+#	- dhcpprofilvar : variable contenant en retour le nom du profil
+#	- msgvar : variable contenant en retour le message d'erreur
+# Sortie :
+#   - valeur de retour : 1 si ok, 0 si erreur
+#   - dhcpprofilvar : nom du profil trouv dans la base (ou Aucun profil)
+#   - msgvar : message d'erreur ventuel
+#
+# Historique
+#   2005/04/08 : pda/jean : conception
+#
+
+proc check-iddhcpprofil {dbfd iddhcpprofil dhcpprofilvar msgvar} {
+    upvar $dhcpprofilvar dhcpprofil
+    upvar $msgvar msg
+
+    set msg ""
+
+    if {! [regexp -- {^[0-9]+$} $iddhcpprofil]} then {
+	set msg "Syntaxe invalide pour le profil DHCP"
+    } else {
+	if {$iddhcpprofil != 0} then {
+	    set sql "SELECT nom FROM dhcpprofil
+				WHERE iddhcpprofil = $iddhcpprofil"
+	    set msg "Profil DHCP invalide ($iddhcpprofil)"
+	    pg_select $dbfd $sql tab {
+		set dhcpprofil $tab(nom)
+		set msg ""
+	    }
+	} else {
+	    set dhcpprofil "Aucun profil"
+	}
+    }
+
+    return [string equal $msg ""]
+}
+
 ##############################################################################
 # Validation d'un domaine
 ##############################################################################
@@ -1765,6 +2025,41 @@ proc valide-role-mail {dbfd idcor nom domaine tabrr tabrrh} {
     return ""
 }
 
+#
+# Valide qu'aucune adresse IP n'empite sur un intervalle DHCP dynamique
+# si l'adresse MAC n'est pas vide.
+#
+# Entre :
+#   - paramtres :
+#       - dbfd : accs  la base
+#	- mac : l'adresse MAC (vide ou non)
+#	- lip : liste des adresses IP (v4 et v6)
+# Sortie :
+#   - valeur de retour : chane vide (ok) ou non vide (message d'erreur)
+#
+# Historique
+#   2004/08/04 : pda/jean : conception
+#
+
+proc valide-dhcp-statique {dbfd mac lip} {
+    set r ""
+    if {! [string equal $mac ""]} then {
+	foreach ip $lip {
+	    set sql "SELECT min, max
+			    FROM dhcprange
+			    WHERE '$ip' >= min AND '$ip' <= max"
+	    pg_select $dbfd $sql tab {
+		set r "$ip est dans l'intervalle DHCP \[$tab(min)..$tab(max)\]"
+	    }
+	    if {! [string equal $r ""]} then {
+		break
+	    }
+	}
+    }
+
+    return $r
+}
+
 ##############################################################################
 # Validation des correspondants
 ##############################################################################
@@ -1867,6 +2162,37 @@ proc lire-hinfo {dbfd texte} {
     return $idhinfo
 }
 
+##############################################################################
+# Validation des dhcpprofil
+##############################################################################
+
+#
+# Lit l'indice du dhcpprofil dans la table
+#
+# Entre :
+#   - dbfd : accs  la base
+#   - texte : texte dhcpprofil  chercher ou ""
+# Sortie :
+#   - valeur de retour : indice, ou 0 si "", ou -1 si non trouv
+#
+# Historique
+#   2005/04/11 : pda/jean : conception
+#
+
+proc lire-dhcpprofil {dbfd texte} {
+    if {[string equal $texte ""]} then {
+	set iddhcpprofil 0
+    } else {
+	set qtexte [::pgsql::quote $texte]
+	set sql "SELECT iddhcpprofil FROM dhcpprofil WHERE nom = '$qtexte'"
+	set iddhcpprofil -1
+	pg_select $dbfd $sql tab {
+	    set iddhcpprofil $tab(iddhcpprofil)
+	}
+    }
+    return $iddhcpprofil
+}
+
 ##############################################################################
 # Rcupration d'informations pour les menus
 ##############################################################################
@@ -1902,6 +2228,91 @@ proc menu-hinfo {dbfd champ defval} {
     return [::webapp::form-menu $champ 1 0 $lhinfo [list $defindex]]
 }
 
+#
+# Rcupre les profils DHCP accessibles par le groupe sous forme d'un
+# menu visible, ou un champ cach si le groupe n'a accs  aucun profil
+# DHCP.
+#
+# Entre :
+#   - dbfd : accs  la base
+#   - champ : champ de formulaire (variable du CGI suivant)
+#   - idcor : identification du correspondant
+#   - iddhcpprofil : identification du profil  slectionner (le profil
+#	pr-existant) ou 0
+# Sortie :
+#   - valeur de retour : code HTML prt  l'emploi
+#
+# Historique
+#   2005/04/08 : pda/jean : conception
+#
+
+proc menu-dhcpprofil {dbfd champ idcor iddhcpprofil} {
+    #
+    # Rcuprer les profils DHCP visibles par le groupe
+    # ainsi que le profil DHCP pr-existant
+    #
+
+    set sql "SELECT p.iddhcpprofil, p.nom
+		FROM dr_dhcpprofil dr, dhcpprofil p, corresp c
+		WHERE c.idcor = $idcor
+		    AND dr.idgrp = c.idgrp
+		    AND dr.iddhcpprofil = p.iddhcpprofil
+		ORDER BY dr.tri ASC, p.nom"
+    set lprof {}
+    set lsel {}
+    set idx 1
+    pg_select $dbfd $sql tab {
+	lappend lprof [list $tab(iddhcpprofil) $tab(nom)]
+	if {$tab(iddhcpprofil) == $iddhcpprofil} then {
+	    lappend lsel $idx
+	}
+	incr idx
+    }
+
+    #
+    # A-t'on trouv au moins un profil ?
+    #
+
+    if {[llength $lprof] > 0} then {
+	#
+	# Est-ce que le profil pr-existant est bien dans notre
+	# liste ?
+	#
+
+	if {$iddhcpprofil != 0 && [llength $lsel] == 0} then {
+	    #
+	    # Non. On va donc ajouter  la fin de la liste
+	    # le profil pr-existant
+	    #
+	    set sql "SELECT iddhcpprofil, nom
+			    FROM dhcpprofil
+			    WHERE iddhcpprofil = $iddhcpprofil"
+	    pg_select $dbfd $sql tab {
+		lappend lprof [list $tab(iddhcpprofil) $tab(nom)]
+		lappend lsel $idx
+	    }
+	}
+
+	#
+	# Ajouter le cas spcial en dbut de liste
+	#
+
+	set lprof [linsert $lprof 0 {0 {Aucun profil}}]
+
+	set html "Profil DHCP&nbsp;: "
+	append html [::webapp::form-menu iddhcpprofil 1 0 $lprof $lsel]
+
+    } else {
+	#
+	# Aucun profil trouv. On cache l'information
+	#
+
+	set html "<INPUT TYPE=HIDDEN NAME=\"$champ\" VALUE=\"$iddhcpprofil\">"
+    }
+
+    return $html
+}
+
 #
 # Fournit le code HTML pour une slection de liste de domaines, soit
 # sous forme de menus droulants si le nombre de domaines autoriss
@@ -2005,7 +2416,8 @@ proc couple-domaine-par-corresp {dbfd idcor where} {
 
 #
 # Fournit du code HTML pour chaque groupe d'informations associ  un
-# groupe - les rseaux, les droits hors rseaux, et les domaines
+# groupe - les rseaux, les droits hors rseaux, les domaines et les
+# profils DHCP.
 #
 # Entre :
 #   - paramtres :
@@ -2014,11 +2426,12 @@ proc couple-domaine-par-corresp {dbfd idcor where} {
 #   - variable globale libconf(tabreseaux) : spc. de tableau
 #   - variable globale libconf(tabdomaines) : spc. de tableau
 # Sortie :
-#   - valeur de retour : liste  trois lments, chaque lment tant
+#   - valeur de retour : liste  4 lments, chaque lment tant
 #	le code HTML associ.
 #
 # Historique
 #   2002/05/23 : pda/jean : spcification et conception
+#   2005/04/06 : pda      : ajout des profils dhcp
 #
 
 proc info-groupe {dbfd idgrp} {
@@ -2031,19 +2444,22 @@ proc info-groupe {dbfd idgrp} {
     set donnees {}
     set sql "SELECT r.idreseau,
 			r.nom, r.localisation, r.adr4, r.adr6,
+			d.dhcp, d.acl,
 			e.nom AS etabl,
 			c.nom AS commu
-		FROM reseau r, plage p, etablissement e, communaute c
-		WHERE p.idgrp = $idgrp
-			AND p.idreseau = r.idreseau
+		FROM reseau r, dr_reseau d, etablissement e, communaute c
+		WHERE d.idgrp = $idgrp
+			AND d.idreseau = r.idreseau
 			AND e.idetabl = r.idetabl
 			AND c.idcommu = r.idcommu
-		ORDER BY r.adr4, r.adr6"
+		ORDER BY d.tri, r.adr4, r.adr6"
     pg_select $dbfd $sql tab {
 	set r_nom 	[::webapp::html-string $tab(nom)]
 	set r_loc	[::webapp::html-string $tab(localisation)]
 	set r_etabl	$tab(etabl)
 	set r_commu	$tab(commu)
+	set r_dhcp	$tab(dhcp)
+	set r_acl	$tab(acl)
 
 	# affadr : utilis pour l'affichage cosmtique des adresses
 	set affadr {}
@@ -2065,6 +2481,13 @@ proc info-groupe {dbfd idgrp} {
 				Communaut $r_commu]
 
 	set droits {}
+
+	set dres {}
+	if {$r_dhcp} then { lappend dres "dhcp" }
+	if {$r_acl} then { lappend dres "acl" }
+	if {[llength $dres] > 0} then {
+	    lappend droits [join $dres ", "]
+	}
 	set sql2 "SELECT adr, allow_deny
 			FROM dr_ip
 			WHERE ($where)
@@ -2091,20 +2514,20 @@ proc info-groupe {dbfd idgrp} {
 
     set donnees {}
     set trouve 0
-    set sql "SELECT adr, allow_deny \
-		    FROM dr_ip \
-		    WHERE NOT (adr <<= ANY ( \
-				SELECT r.adr4 \
-					FROM reseau r, plage p \
-					WHERE r.idreseau = p.idreseau \
-						AND p.idgrp = $idgrp \
+    set sql "SELECT adr, allow_deny
+		    FROM dr_ip
+		    WHERE NOT (adr <<= ANY (
+				SELECT r.adr4
+					FROM reseau r, dr_reseau d
+					WHERE r.idreseau = d.idreseau
+						AND d.idgrp = $idgrp
 				UNION
-				SELECT r.adr6 \
-					FROM reseau r, plage p \
-					WHERE r.idreseau = p.idreseau \
-						AND p.idgrp = $idgrp \
-				    ) ) \
-			AND idgrp = $idgrp \
+				SELECT r.adr6
+					FROM reseau r, dr_reseau d
+					WHERE r.idreseau = d.idreseau
+						AND d.idgrp = $idgrp
+				    ) )
+			AND idgrp = $idgrp
 		    ORDER BY adr"
     set droits {}
     pg_select $dbfd $sql tab {
@@ -2148,49 +2571,81 @@ proc info-groupe {dbfd idgrp} {
 
 	lappend donnees [list Domaine $tab(nom) $rm $rw]
     }
-    set tabdomaines [::arrgen::output "html" $libconf(tabdomaines) $donnees]
+    if {[llength $donnees] > 0} then {
+	set tabdomaines [::arrgen::output "html" $libconf(tabdomaines) $donnees]
+    } else {
+	set tabdomaines "Aucun profil DHCP actif"
+    }
 
-    return [list $tabreseaux $tabcidrhorsreseau $tabdomaines]
+    #
+    # Slectionner les profils DHCP
+    #
+
+    set donnees {}
+    set sql "SELECT p.nom, dr.tri, p.texte \
+			FROM dhcpprofil p, dr_dhcpprofil dr
+			WHERE p.iddhcpprofil = dr.iddhcpprofil \
+				AND dr.idgrp = $idgrp \
+			ORDER BY dr.tri, p.nom"
+    pg_select $dbfd $sql tab {
+	lappend donnees [list DHCP $tab(nom) $tab(texte)]
+    }
+    if {[llength $donnees] > 0} then {
+	set tabdhcpprofil [::arrgen::output "html" $libconf(tabdhcpprofil) $donnees]
+    } else {
+	set tabdhcpprofil "Aucun profil DHCP autoris"
+    }
+
+    return [list $tabreseaux $tabcidrhorsreseau $tabdomaines $tabdhcpprofil]
 }
 
 #
-# Fournit du code HTML pour une listbox contenant soit les rseaux associs
-# au groupe, soit tous les rseaux en slectionnant ceux associs au groupe.
+# Fournit la liste des rseaux associs  un groupe avec un certain droit.
 #
 # Entre :
 #   - paramtres :
 #	- dbfd : accs  la base
 #	- idgrp : identificateur du groupe
-#	- champ : nom du champ de formulaire gnr pour la listbox
-#	- mode : "tous" ou "groupe"
+#	- droit : "consult", "dhcp" ou "acl"
 # Sortie :
-#   - valeur de retour : code HTML de la listbox
+#   - valeur de retour : liste des rseaux sous la forme {idreseau nom-complet}
 #
 # Historique
 #   2004/01/16 : pda/jean : spcification et conception
+#   2004/08/06 : pda/jean : extension des droits sur les rseaux
+#   2004/10/05 : pda/jean : adaptation aux nouveaux droits
 #
 
-proc liste-reseaux {dbfd idgrp mode champ hauteur} {
+proc liste-reseaux {dbfd idgrp droit} {
     #
-    # Construire une clause "where" si on ne rcupre que les rseau
-    # auxquels le groupe a accs.
+    # Mettre en forme les droits pour la clause where
     #
 
-    set where ""
-    if {[string equal $mode "groupe"]} then {
-	set where "WHERE idreseau IN (SELECT idreseau
-					FROM plage WHERE idgrp = $idgrp)"
+    switch -- $droit {
+	consult {
+	    set w1 ""
+	    set w2 ""
+	}
+	dhcp {
+	    set w1 "AND d.$droit > 0"
+	    set w2 "AND r.$droit > 0"
+	}
+	acl {
+	    set w1 "AND d.$droit > 0"
+	    set w2 ""
+	}
     }
 
     #
-    # Rcuprer tous les rseaux (ventuellement filtrs par le groupe)
+    # Rcuprer tous les rseaux autoriss par le groupe selon ce droit
     #
 
     set lres {}
-    set lidreseau {}
-    set sql "SELECT idreseau, nom, adr4, adr6
-			FROM reseau
-			$where
+    set sql "SELECT r.idreseau, r.nom, r.adr4, r.adr6
+			FROM reseau r, dr_reseau d
+			WHERE r.idreseau = d.idreseau
+			    AND d.idgrp = $idgrp
+			    $w1 $w2
 			ORDER BY adr4, adr6"
     pg_select $dbfd $sql tab {
 	lappend lres [list $tab(idreseau) \
@@ -2200,39 +2655,112 @@ proc liste-reseaux {dbfd idgrp mode champ hauteur} {
 					[::webapp::html-string $tab(nom)] \
 				    ] \
 			    ]
-	lappend lidreseau $tab(idreseau)
+    }
+
+    return $lres
+}
+
+#
+# Valide un idreseau tel que retourn par un formulaire. Cette validation
+# est ralis dans le contexte d'un groupe, avec test d'un droit particulier.
+#
+# Entre :
+#   - paramtres :
+#	- dbfd : accs  la base
+#	- idreseau : id  vrifier
+#	- idgrp : identificateur du groupe
+#	- droit : "consult", "dhcp" ou "acl"
+#	- version : 4, 6 ou {4 6}
+#	- msgvar : message d'erreur en retour
+# Sortie :
+#   - valeur de retour : liste de CIDR, ou liste vide
+#   - paramtre msgvar : message d'erreur en retour si liste vide
+#
+# Historique
+#   2004/10/05 : pda/jean : spcification et conception
+#
+
+proc valide-idreseau {dbfd idreseau idgrp droit version msgvar} {
+    upvar $msgvar msg
+
+    #
+    # Valider le numro de rseau au niveau syntaxique
+    #
+    set idreseau [string trim $idreseau]
+    if {! [regexp {^[0-9]+$} $idreseau]} then {
+	set msg "Plage rseau invalide ($idreseau)"
+	return {}
     }
 
     #
-    # Slectionner les rseaux du groupe
+    # Convertir le droit en clause where
     #
 
-    set lsel {}
-    if {[string equal $mode "tous"]} then {
-	set lid [::pgsql::getcols $dbfd plage "idgrp=$idgrp" "" idreseau]
-	foreach idreseau $lid {
-	    set idx [lsearch -exact $lidreseau $idreseau]
-	    if {$idx != -1} then {
-		lappend lsel $idx
-	    }
+    switch -- $droit {
+	consult {
+	    set w1 ""
+	    set w2 ""
+	    set c "en consultation"
+	}
+	dhcp {
+	    set w1 "AND d.$droit > 0"
+	    set w2 "AND r.$droit > 0"
+	    set c "pour le droit '$droit'"
+	}
+	acl {
+	    set w1 "AND d.$droit > 0"
+	    set w2 ""
+	    set c "pour le droit '$droit'"
 	}
     }
 
     #
-    # Gnrer le code HTML
+    # Valider le numro de rseau et rcuprer le ou les CIDR associ(s)
     #
 
-    set taille [llength $lres]
-    if {$taille == 0} then {
-	set listereseaux "Aucune plage slectionnable"
-    } else {
-	if {$taille > $hauteur} then {
-	    set taille $hauteur
-	}
-	set listereseaux [::webapp::form-menu $champ $taille 1 $lres $lsel]
+    set lcidr {}
+    set msg ""
+
+    set sql "SELECT r.adr4, r.adr6
+		    FROM dr_reseau d, reseau r
+		    WHERE d.idgrp = $idgrp
+			AND d.idreseau = r.idreseau
+			AND r.idreseau = $idreseau
+			$w1 $w2"
+    set cidrplage4 ""
+    set cidrplage6 ""
+    pg_select $dbfd $sql tab {
+	set cidrplage4 $tab(adr4)
+	set cidrplage6 $tab(adr6)
     }
 
-    return $listereseaux
+    if {[lsearch -exact $version 4] == -1} then {
+	set cidrplage4 ""
+    }
+    if {[lsearch -exact $version 6] == -1} then {
+	set cidrplage6 ""
+    }
+
+    set vide4 [string equal $cidrplage4 ""]
+    set vide6 [string equal $cidrplage6 ""]
+
+    switch -glob $vide4-$vide6 {
+	1-1 {
+	    set msg "Vous n'avez pas accs  ce rseau $c"
+	}
+	0-1 {
+	    lappend lcidr $cidrplage4
+	}
+	1-0 {
+	    lappend lcidr $cidrplage6
+	}
+	0-0 {
+	    lappend lcidr $cidrplage4
+	    lappend lcidr $cidrplage6
+	}
+    }
+
+    return $lcidr
 }
 
 ##############################################################################
